@@ -20,6 +20,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+import os
 import unittest
 import hashlib
 from threading import Thread
@@ -58,13 +59,16 @@ def myauth(username, password, useragent=None):
            password == hashlib.md5('s3cr3t'.encode()).hexdigest() and \
            useragent == 'AuthRPC_unittest'
 
-def make_server():
+def mybadauth(username, password, useragent=None):
+    return 1 / 0  # generate exception
+
+def make_server(api, auth, port):
     from server import AuthRPCApp
     class myhandler(simple_server.WSGIRequestHandler):
         def log_request(self, *a, **b):
             pass # do not output log messages
-    application = AuthRPCApp(api(), auth=myauth)
-    return simple_server.make_server('localhost', 1337, application, handler_class=myhandler)
+    application = AuthRPCApp(api, auth, filepath='')
+    return simple_server.make_server('localhost', port, application, handler_class=myhandler)
 ##### server ^^^ #####
 
 ##### client vvv #####
@@ -92,6 +96,22 @@ class AuthTest(unittest.TestCase):
         with self.assertRaises(UnauthorisedError):
             self.client.api.mymethod()
 
+class BadAuthTest(unittest.TestCase):
+    def runTest(self):
+        from client import ServerProxy, RemoteException
+        self.client = ServerProxy('http://localhost:1338/')
+        with self.assertRaises(RemoteException):
+            self.client.mymethod()
+
+class BadBatchAuthTest(unittest.TestCase):
+    def runTest(self):
+        from client import ServerProxy, RemoteException, BatchCall
+        self.client = ServerProxy('http://localhost:1338/')
+        batch = BatchCall(self.client)
+        batch.api.echo('One')
+        batch.api.echo(mystring='Two')
+        with self.assertRaises(RemoteException):
+            batch()
 
 @unittest.skipIf(NO_INTERNET, 'http://www.wyre-it.co.uk/ not contactable')
 class NotFoundTest(unittest.TestCase):
@@ -158,11 +178,25 @@ class ProtocolErrorTest(AuthRPCTests):
             self.client.api.test(1, '2', three=3)
 
 class FileTest(AuthRPCTests):
+    def setUp(self):
+        AuthRPCTests.setUp(self)
+
+        # create big (100Mb) file
+        self.tempfile = 'bigfile.tmp'
+        with open(self.tempfile,'w') as f:
+            for x in range(1024*100):
+                f.write('x'*1024) # 1k of data
+
     def runTest(self):
-        filename = 'LICENCE.txt' 
-        with open(filename,'rb') as f:
-            source = f.read()
-        self.assertEqual(self.client.__getfile__(filename), source)
+        with open(self.tempfile,'rb') as f:
+            servercopy = self.client.__getfile__(self.tempfile)
+            for data in servercopy:
+                source = f.read(len(data))
+                self.assertEqual(data, source)
+
+    def tearDown(self):
+        AuthRPCTests.tearDown(self)
+        os.remove(self.tempfile)
 
 class NonExistentFileTest(AuthRPCTests):
     def runTest(self):
@@ -208,14 +242,24 @@ class SetFinishedFlag(unittest.TestCase):
 def suite():
     global finished
     finished = False
-    # create server
+
+    # create server 1
     def test_wrapper():
-        server = make_server()
+        server = make_server(api(), myauth, 1337)
         while not finished:
             server.handle_request()
     thread = Thread(target=test_wrapper)
     thread.start()
-    time.sleep(0.1) # wait for server thread to start
+    
+    # create server 2
+    def test_wrapper():
+        server = make_server(api(), mybadauth, 1338)
+        while not finished:
+            server.handle_request()
+    thread = Thread(target=test_wrapper)
+    thread.start()
+
+    time.sleep(0.1) # wait for server threads to start
 
     # tests are as client
     suite = unittest.TestSuite()
@@ -232,9 +276,11 @@ def suite():
     suite.addTest(FileTest())
     suite.addTest(NonExistentFileTest())
     suite.addTest(BadAuthFileTest())
+    suite.addTest(BadAuthTest())
     suite.addTest(BatchTest())
     suite.addTest(SetFinishedFlag())
     suite.addTest(BadBatchTest())
+    suite.addTest(BadBatchAuthTest())
     # btc fixme - test a list/tuple of api classes in another server
     return suite
 
